from .base import BaseAdapter
import requests


class IonomyAdapter(BaseAdapter):

    BASE_URL = "https://ionomy.com/api/v1/public/orderbook"

    def get_price(self):
        btc_hive_response = requests.get(
            self.BASE_URL,
            params={"market": "btc-hive", "type": "asks"})

        btc_hive_response_json = self.validate_response(btc_hive_response)

        usd_btc_response =requests.get(
            "https://api.coingecko.com/api/v3/simple/price?ids="
            "bitcoin&vs_currencies=usd"
        )
        usd_btc_response_json = self.validate_response(usd_btc_response)

        if btc_hive_response_json and usd_btc_response_json:
            btc_hive_pair = btc_hive_response_json["data"]["asks"][0]["price"]
            usd_btc_pair = usd_btc_response_json["bitcoin"]["usd"]

            price = round(usd_btc_pair * float(btc_hive_pair), self.PRICE_PRECISION)
            self.logger.info(self.bundle_adapter_name(f'Price: {price}'))
            return price



from .base import BaseAdapter
import requests


class BittrexAdapter(BaseAdapter):

    BASE_URL = "https://bittrex.com/api/v1.1/public/getticker"

    def get_price(self):
        btc_hive_response = requests.get(
            self.BASE_URL, params={"market": "BTC-HIVE"})
        btc_hive_response_json = self.validate_response(btc_hive_response)

        usd_btc_response = requests.get(
            self.BASE_URL, params={"market": "USD-BTC"}
        )
        usd_btc_response_json = self.validate_response(usd_btc_response)

        if btc_hive_response_json and usd_btc_response_json:
            btc_hive_pair = btc_hive_response_json["result"]["Last"]
            usd_btc_pair = usd_btc_response_json["result"]["Last"]

            price = round(usd_btc_pair * btc_hive_pair, self.PRICE_PRECISION)
            self.logger.info(self.bundle_adapter_name(f'Price: {price}'))
            return price



from .base import BaseAdapter
import requests


class ProbitAdapter(BaseAdapter):

    BASE_URL = "https://api.probit.com/api/exchange/v1/ticker"

    def get_price(self):
        ticker_data = requests.get(
            self.BASE_URL,
            params={"market_ids": "BTC-USDT,HIVE-BTC"})

        ticker_response = self.validate_response(ticker_data).get("data", [])

        usd_btc_response = [
            r for r in ticker_response if r["market_id"] == "BTC-USDT"][0]

        btc_hive_response = [
            r for r in ticker_response if r["market_id"] == "HIVE-BTC"][0]

        btc_hive_pair = btc_hive_response["last"]
        usd_btc_pair = usd_btc_response["last"]

        price = round(float(usd_btc_pair) * float(btc_hive_pair), self.PRICE_PRECISION)
        self.logger.info(self.bundle_adapter_name(f'Price: {price}'))
        return price



from .base import BaseAdapter
import requests


class BinanceAdapter(BaseAdapter):

    BASE_URL = "https://api.binance.com/api/v3/ticker/price"

    def get_price(self):
        hive_usdt_response = requests.get(
            self.BASE_URL, params={"symbol": "HIVEUSDT"}

        )
        hive_usdt_response_json = self.validate_response(hive_usdt_response)

        if hive_usdt_response_json:
            hive_usdt_price = float(hive_usdt_response_json["price"])
            price = round(hive_usdt_price, self.PRICE_PRECISION)
            self.logger.info(self.bundle_adapter_name(f'Price: {price}'))
            return price



import os

import numpy as np

from .adapters.bittrex import BittrexAdapter
from .adapters.probit import ProbitAdapter
from .adapters.huobi import HuobiAdapter
from .adapters.ionomy import IonomyAdapter
from .adapters.mxc import MxcAdapter
from .adapters.binance import BinanceAdapter

AVAILABLE_MARKETS = {
    "bittrex": BittrexAdapter,
    "probit": ProbitAdapter,
    "huobi": HuobiAdapter,
    "ionomy": IonomyAdapter,
    "mxc": MxcAdapter,
    "binance": BinanceAdapter,
}

# Don't set MXC as a default market
# since it requires an API key and it has a TTL of 90 days.
DEFAULT_MARKETS = list(set(AVAILABLE_MARKETS.keys()) - set(["mxc"]))


def is_marketlist_valid(market_list):
    # that's a crazy one liner.
    # it checks the given market list's all elements are defined in implemented
    # markets.
    return len(set(market_list).difference(set(AVAILABLE_MARKETS.keys()))) == 0


def get_prices(markets):
    config_options = {
        "MXC_API_KEY": os.getenv("MXC_API_KEY")  # Only needed for MXC Exchange
    }
    return [
        AVAILABLE_MARKETS.get(market)(
            config=config_options).get_price() for market in markets]


def get_average_price(prices):
    return round(sum(prices) / len(prices), 3)


def exclude_outliers(prices, threshold=35):
    """
    An outlier logic to exclude bad markets.
    Uses a slightly modified version of "Modified Z-score".
    https://www.itl.nist.gov/div898/handbook/eda/section3/eda35h.htm
    """
    if isinstance(prices, list):
        prices = np.array(prices)
    original_median = np.median(prices)
    deviation = np.abs(prices - original_median)
    median_deviation = np.median(deviation)
    score = 0
    if median_deviation:
        score = deviation / median_deviation
    filtered_prices = prices[score < threshold]
    return list(filtered_prices)



from setuptools import setup

from transmitter import __version__

setup(
    name='transmitter',
    version=__version__,
    packages=[
        'transmitter',
        'transmitter.pricefeed',
        'transmitter.pricefeed.adapters',
    ],
    url='http://github.com/emre/transmitter',
    license='MIT',
    author='emre yilmaz',
    author_email='mail@emreyilmaz.me',
    description='HIVE Witness updates made easy.',
    entry_points={
        'console_scripts': [
            'transmitter = transmitter.main:main',
        ],
    },
    install_requires=["beem==0.24.26", "requests", "numpy"]
)
